export const mapSzX = 12;
export const mapSzY= 9;

export function checkGrid<T>(cells: Array<Array<T>>): Array<Array<T>> {
  if (cells.length != mapSzY) {
    throw `map must be ${mapSzX}x${mapSzY}`
  }

  for (let row of cells.values()) {
    if (row.length != mapSzX) {
      throw `map must be ${mapSzX}x${mapSzY}`;
    }
  }
  return cells;
}

export type ConceptualCell = "X" | "." | " ";

function loadMap(map: Array<string>): Array<Array<ConceptualCell>> {
  let newRows: Array<Array<ConceptualCell>> = [];
  for (let oldRow of map.values()) {
    let newRow: Array<ConceptualCell> = []
    for (let i = 0; i < oldRow.length; i++) {
      let char = oldRow.charAt(i);
      let cell: ConceptualCell
      switch(char) {
        case "X": cell = "X"; break;
        case ".": cell = "."; break;
        case " ": cell = " "; break;
        default:
          throw `map element not valid: ${char}`
      }
      newRow.push(cell);
    }
    newRows.push(newRow);
  }
  return checkGrid(newRows);
}

export let maps: Record<string, Array<Array<ConceptualCell>>> = {
  map0: loadMap([
    "XX        XX",
    "X.        .X",
    "            ",
    "            ",
    "            ",
    "            ",
    "            ",
    "X.        .X",
    "XX        XX",
  ]),
  map1: loadMap([
    "XX.       XX",
    "XXX    X  .X",
    ".XX    XX   ",
    "       .XX  ",
    "            ",
    "  XX.       ",
    "   XX    XX.",
    "X.  X    XXX",
    "XX       .XX",
  ]),
  map2: loadMap([
    "XX  XXXX  XX",
    "X. XXXXXX .X",
    "   XX..XX   ",
    "            ",
    "            ",
    "            ",
    "   XX..XX   ",
    "X. XXXXXX .X",
    "XX  XXXX  XX",
  ]),
  map3: loadMap([
    "X   XXXX   X",
    " .        . ",
    "   .    .   ",
    "X   XXXX   X",
    "X   XXXX   X",
    "X   XXXX   X",
    "   .    .   ",
    " .        . ",
    "X   XXXX   X",
  ]),
}
